import { resolve, dirname } from "path";
import { createReadStream, createWriteStream } from "fs";
import { createHash } from "crypto";
import { getRandomId } from "dr-js/module/common/math/random";
import { createCacheMap } from "dr-js/module/common/data/CacheMap";
import { toString as arrayBufferToString, fromString } from "dr-js/module/common/data/ArrayBuffer";
import { packChainArrayBufferPacket, parseChainArrayBufferPacket } from "dr-js/module/common/data/ArrayBufferPacket";
import { createAsyncTaskQueue } from "dr-js/module/common/module/AsyncTaskQueue";
import { pipeStreamAsync } from "dr-js/module/node/data/Stream";
import { toArrayBuffer } from "dr-js/module/node/data/Buffer";
import { createPathPrefixLock, writeFileAsync } from "dr-js/module/node/file/function";
import { createDirectory } from "dr-js/module/node/file/File";
import { modify } from "dr-js/module/node/file/Modify";
const createFileChunkUpload = async ({rootPath, mergePath, onError, expireTime = 6e5, chunkCacheMap = (() => createCacheMap({
  valueSizeSumMax: 64
}))()}) => {
  await createDirectory(rootPath);
  await createDirectory(mergePath);
  const getPath = createPathPrefixLock(rootPath);
  const {pushTask} = createAsyncTaskQueue(onError);
  chunkCacheMap.subscribe(({type, key, payload}) => {
    if ("delete" !== type) return;
    const {tempPath} = payload;
    pushTask(() => modify.delete(tempPath).catch(() => {}));
  });
  return async (bufferPacket, cacheKeyPrefix) => {
    const [headerArrayBuffer, chunkHashArrayBuffer, chunkArrayBuffer] = parseChainArrayBufferPacket(toArrayBuffer(bufferPacket));
    const {filePath: relativePath, chunkByteLength, chunkIndex, chunkTotal} = JSON.parse(arrayBufferToString(headerArrayBuffer));
    const chunkBuffer = Buffer.from(chunkArrayBuffer);
    if (chunkByteLength !== chunkBuffer.length) throw new Error(`chunk length mismatch, get: ${chunkBuffer.length}, expect ${chunkByteLength}`);
    if (chunkHashArrayBuffer.byteLength) {
      const chunkHashBuffer = Buffer.from(chunkHashArrayBuffer);
      const verifyChunkHashBuffer = createHash("sha256").update(chunkBuffer).digest();
      if (0 !== Buffer.compare(chunkHashBuffer, verifyChunkHashBuffer)) throw new Error(`chunk hash mismatch, get: ${verifyChunkHashBuffer.toString("base64")}, expect ${chunkHashBuffer.toString("base64")}`);
    }
    const cacheKey = `${cacheKeyPrefix}-${relativePath}-${chunkTotal}`;
    let chunkData = chunkCacheMap.get(cacheKey);
    if (void 0 === chunkData) {
      const filePath = getPath(relativePath);
      const tempPath = resolve(mergePath, getRandomId(cacheKey).replace(/[^\w-.]/g, "_"));
      await createDirectory(tempPath);
      chunkData = {
        tempPath,
        filePath,
        chunkTotal,
        chunkList: []
      };
    }
    const chunkPath = resolve(chunkData.tempPath, `chunk-${chunkIndex}-${chunkTotal}`);
    await writeFileAsync(chunkPath, chunkBuffer);
    chunkData.chunkList[chunkIndex] = {
      chunkIndex,
      chunkPath
    };
    const chunkCacheCount = Object.keys(chunkData.chunkList).length;
    if (chunkCacheCount === chunkTotal) {
      await createDirectory(dirname(chunkData.filePath));
      await writeFileAsync(chunkData.filePath, "");
      for (const {chunkPath} of chunkData.chunkList) await pipeStreamAsync(createWriteStream(chunkData.filePath, {
        flags: "a"
      }), createReadStream(chunkPath));
      await modify.delete(chunkData.tempPath);
      chunkCacheMap.delete(cacheKey);
    } else chunkCacheCount > 1 ? chunkCacheMap.touch(cacheKey, Date.now() + expireTime) : chunkCacheMap.set(cacheKey, chunkData, 1, Date.now() + expireTime);
  };
};
const uploadFileByChunk = async ({fileBuffer, filePath, uploadFileChunkBuffer, onProgress, chunkSizeMax = 1048576}) => {
  const fileSize = fileBuffer.length;
  const chunkBufferList = ((fileBuffer, chunkSizeMax = 1048576) => {
    const fileSize = fileBuffer.length;
    const chunkTotal = Math.ceil(fileSize / chunkSizeMax) || 1;
    const chunkBufferList = [];
    let chunkIndex = 0;
    for (;chunkIndex < chunkTotal; ) {
      const chunkSize = chunkIndex < chunkTotal - 1 ? chunkSizeMax : fileSize % chunkSizeMax;
      const chunkBuffer = fileBuffer.slice(chunkIndex * chunkSizeMax, chunkIndex * chunkSizeMax + chunkSize);
      chunkBufferList.push({
        chunkBuffer,
        chunkIndex,
        chunkTotal
      });
      chunkIndex += 1;
    }
    return chunkBufferList;
  })(fileBuffer, chunkSizeMax);
  for (const {chunkBuffer, chunkIndex, chunkTotal} of chunkBufferList) {
    onProgress && onProgress(chunkIndex * chunkSizeMax, fileSize);
    const chunkArrayBuffer = toArrayBuffer(chunkBuffer);
    const chunkByteLength = chunkArrayBuffer.byteLength;
    const verifyChunkHashBuffer = createHash("sha256").update(chunkBuffer).digest();
    const chunkInfo = {
      filePath,
      chunkByteLength,
      chunkIndex,
      chunkTotal
    };
    await uploadFileChunkBuffer(Buffer.from(packChainArrayBufferPacket([ fromString(JSON.stringify(chunkInfo)), toArrayBuffer(verifyChunkHashBuffer), chunkArrayBuffer ])), chunkInfo);
  }
  onProgress && onProgress(fileSize, fileSize);
};
export { createFileChunkUpload, uploadFileByChunk };